﻿using System;
using System.Collections.Concurrent;
using log4net;


namespace ProducerConsumerGeneric
{
    /// <summary>
    /// Consumer in a Producer-Consumer framework
    /// </summary>
    /// <typeparam name="T">The type this consumer consumes</typeparam>
    public class Consumer<T>
    {
        private readonly BlockingCollection<T> _buffer;
        private  readonly ILog _logger = LogManager.GetLogger("producerconsumer");

        /// <summary>
        /// Constructs a new object of this class
        /// </summary>
        /// <param name="buffer">the buffer to use</param>
        /// <exception cref="ArgumentNullException">if buffer is null</exception>
        public Consumer(BlockingCollection<T> buffer)
        {
            if (buffer == null) { throw new ArgumentNullException("buffer"); }
            _buffer = buffer;
        }

        /// <summary>
        /// Runs the Consumer object
        /// </summary>
        public void Run()
        {
            try
            {
                while (!_buffer.IsCompleted)
                {
                    T element = _buffer.Take();
                    //Console.WriteLine("Consumer: " + element);
                    _logger.Info("Consumer: " + element);
                }
            }
            catch (InvalidOperationException) { /* ignored */ }
            Console.WriteLine("Consumer done");
        }
    }
}
