﻿using System;
using System.Collections.Concurrent;

namespace ProducerConsumerGeneric
{
    /// <summary>
    /// MiddleMan in a Producer-Consumer framework
    /// The MiddleMan takes and element from one buffer and add the element to another buffer
    /// </summary>
    /// <typeparam name="T">The type of elements this MiddleMan handles</typeparam>
    public class MiddleMan<T> 
    {
        private readonly BlockingCollection<T> _bufferIn;
        private readonly BlockingCollection<T> _bufferOut;

        /// <summary>
        /// Creates a new object of this class
        /// </summary>
        /// <param name="bufferIn">The ingoing buffer</param>
        /// <param name="bufferOut">The outgoing buffer</param>
        /// <exception cref="ArgumentNullException">if bufferIn or bufferOut is null</exception>
        public MiddleMan(BlockingCollection<T> bufferIn, BlockingCollection<T> bufferOut)
        {
            if (bufferIn == null) { throw new ArgumentNullException("bufferIn"); }
            if (bufferOut == null) { throw new ArgumentNullException("bufferOut"); }
            _bufferIn = bufferIn;
            _bufferOut = bufferOut;
        }

        /// <summary>
        /// Runs this MiddleMan object
        /// </summary>
        public void Run()
        {
            while (!_bufferIn.IsCompleted)
            {
                try
                {
                    T element = _bufferIn.Take();
                    _bufferOut.Add(element);
                    Console.WriteLine("Middleman: " + element);
                }
                catch (InvalidOperationException) { /* ignored */ }
            }
            _bufferOut.CompleteAdding();
        }
    }
}
