﻿using System;
using System.Collections.Concurrent;

namespace ProducerConsumerGeneric
{
    /// <summary>
    /// MiddleMan in a Producer-Consumer framework
    /// The MiddleMan takes and element from one buffer and add the element to another buffer
    /// </summary>  
    /// <typeparam name="TIn">The type of elements this MiddleMan consumes</typeparam>
    /// <typeparam name="TOut">The type of elements this MiddleMan produces</typeparam>
    public class MiddleMan2<TIn, TOut> where TIn : TOut
    {
        private readonly BlockingCollection<TIn> _bufferIn;
        private readonly BlockingCollection<TOut> _bufferOut;

        /// <summary>
        /// Creates a new object of this class
        /// </summary>
        /// <param name="bufferIn">The ingoing buffer</param>
        /// <param name="bufferOut">The outgoing buffer</param>
        /// <exception cref="ArgumentNullException">if bufferIn or bufferOut is null</exception>
        public MiddleMan2(BlockingCollection<TIn> bufferIn, BlockingCollection<TOut> bufferOut)
        {
            if (bufferIn == null) { throw new ArgumentNullException("bufferIn"); }
            if (bufferOut == null) { throw new ArgumentNullException("bufferOut"); }
            _bufferIn = bufferIn;
            _bufferOut = bufferOut;
        }

        /// <summary>
        /// Runs this MiddleMan object
        /// </summary>
        public void Run()
        {
            while (!_bufferIn.IsCompleted)
            {
                try
                {
                    TIn element = _bufferIn.Take();
                    _bufferOut.Add(element);
                    Console.WriteLine("Middleman: " + element);
                }
                catch (InvalidOperationException) { /* ignored */ }
            }
            _bufferOut.CompleteAdding();
        }
    }
}
