﻿using System;
using System.Collections.Concurrent;

namespace ProducerConsumerGeneric
{
    /// <summary>
    /// Producer in a Producer-Consumer framework
    /// </summary>
    /// <typeparam name="T">The type of element that this Producer produces</typeparam>
    public class Producer<T>
    {
        private readonly BlockingCollection<T> _buffer;
        private readonly int _howMany;
        private readonly IElementFactory<T> _factory;

        /// <summary>
        /// Creates a new objec of this class
        /// </summary>
        /// <param name="buffer">The buffer to add to</param>
        /// <param name="howMany">The number of items to produce</param>
        /// <param name="factory">The factory that really produces the elements</param>
        /// <exception cref="ArgumentNullException">if buffer is null or factory is null</exception>
        /// <exception cref="ArgumentOutOfRangeException">if howMany is negative or 0</exception>
        public Producer(BlockingCollection<T> buffer, int howMany, IElementFactory<T> factory)
        {
            if (buffer == null) { throw new ArgumentNullException("buffer"); }
            if (howMany <= 0) { throw new ArgumentOutOfRangeException("howMany"); }
            if (factory == null) { throw new ArgumentNullException("factory"); }
            _buffer = buffer;
            _howMany = howMany;
            _factory = factory;
        }

        /// <summary>
        /// Runs this Producer
        /// </summary>
        public void Run()
        {
            for (int i = 0; i < _howMany; i++)
            {
                T element = _factory.GetElement(i);
                _buffer.Add(element);
                Console.WriteLine("Producer: " + element);
            }
            _buffer.CompleteAdding();
            Console.WriteLine("Producer done");
        }
    }
}
