﻿using System;
using System.Collections.Concurrent;
using System.Threading.Tasks;
using log4net.Config;

namespace ProducerConsumerGeneric
{
    /// <summary>
    /// Some private static methods to try the different configuration of Producer, Consumer and MiddleMan
    /// </summary>
    class Program
    {
        static void Main()
        {
            BasicConfigurator.Configure();
            //SimpleProducerConsumer();
            //ProducerTwoConsumers();
            //ProducerMiddleManConsumer();
            //ProducerMiddleMan2ConsumerSameType();
            ProducerMiddleMan2ConsumerDifferentTypes();
        }

        private static void SimpleProducerConsumer()
        {
            BlockingCollection<String> buffer = new BlockingCollection<string>(5);
            Producer<String> producer = new Producer<string>(buffer, 10, StringFactory.Instance);
            Consumer<String> consumer = new Consumer<String>(buffer);
            Parallel.Invoke(producer.Run, consumer.Run);
        }

        private static void ProducerTwoConsumers()
        {
            BlockingCollection<String> buffer = new BlockingCollection<string>(5);
            Producer<String> producer = new Producer<string>(buffer, 10, StringFactory.Instance);
            Consumer<String> consumer1 = new Consumer<String>(buffer);
            Consumer<String> consumer2 = new Consumer<String>(buffer);
            Parallel.Invoke(producer.Run, consumer1.Run, consumer2.Run);
        }

        private static void ProducerMiddleManConsumer()
        {
            BlockingCollection<String> buffer1 = new BlockingCollection<string>(5);
            BlockingCollection<String> buffer2 = new BlockingCollection<string>(5);

            Producer<String> producer = new Producer<string>(buffer1, 10, StringFactory.Instance);
            MiddleMan<String> middleman = new MiddleMan<string>(buffer1, buffer2);
            Consumer<String> consumer = new Consumer<String>(buffer2);
            Parallel.Invoke(producer.Run, middleman.Run, consumer.Run);
        }

        private static void ProducerMiddleMan2ConsumerSameType()
        {
            BlockingCollection<String> buffer1 = new BlockingCollection<string>(5);
            BlockingCollection<String> buffer2 = new BlockingCollection<string>(5);

            Producer<String> producer = new Producer<string>(buffer1, 10, StringFactory.Instance);
            MiddleMan2<String, string> middleman = new MiddleMan2<string, string>(buffer1, buffer2);
            Consumer<String> consumer = new Consumer<String>(buffer2);
            Parallel.Invoke(producer.Run, middleman.Run, consumer.Run);
        }

        private static void ProducerMiddleMan2ConsumerDifferentTypes()
        {
            BlockingCollection<String> buffer1 = new BlockingCollection<string>(5);
            BlockingCollection<object> buffer2 = new BlockingCollection<object>(5);

            Producer<String> producer = new Producer<string>(buffer1, 10, StringFactory.Instance);
            MiddleMan2<String, object> middleman = new MiddleMan2<string, object>(buffer1, buffer2);
            Consumer<object> consumer = new Consumer<object>(buffer2);
            Parallel.Invoke(producer.Run, middleman.Run, consumer.Run);
        }
    }
}
